# 3.2.0

Date: 2024-08-26
Tag: 3.2.0

## Overview

3.2.0 is the first [stable][release_policy] version of the 3.2 release
series. It introduces 30 improvements and resolves 5 bugs since
3.1.2.

The "stable" label means that we have all planned features implemented and we
see no high-impact issues. However, if you encounter an issue, feel free to
[report it][issues] on GitHub.

[release_policy]: https://www.tarantool.io/en/doc/latest/release/policy/
[issues]: https://github.com/tarantool/tarantool/issues

## Compatibility

Tarantool 3.x is backward compatible with Tarantool 2.11.x in the binary data
layout, client-server protocol, and replication protocol.

Please [upgrade][upgrade] using the `box.schema.upgrade()` procedure to unlock
all the new features of the 3.x series.

[upgrade]: https://www.tarantool.io/en/doc/latest/book/admin/upgrades/

## Functionality added or changed

### Core

* Implemented new iterators for the memtx tree index: 'np' (next prefix)
  and 'pp' (previous prefix) (gh-9994).
* The splice operation (':') in update/upsert is now allowed for varbinary fields
  and can take a varbinary argument for insertion (gh-9997).
* Any 128-byte sequence is considered as a valid UUID value to support all
  RFC 9562 UUID versions (gh-5444).
* `box.commit()` has a new option `wait`. It can be passed as
  `box.commit{wait=...}` and regulates whether the commit must be blocking or
  async (gh-67).
* Added the source tuple as a second argument for the functional variant of the
  default field values (gh-9825).
* Added the `tuple:format` method to get the tuple format (gh-10005).
* Added the `is_interval` function to check that the provided value is
  `interval` (gh-10269).
* A functional index can now use a function that does not have the flag
  `is_sandboxed` (gh-10349).


### Replication

* Added the `age` and `confirm_lag` fields to `box.info.synchro.queue`: the
  former shows the time that the oldest entry currently present in the queue has
  spent waiting for the quorum, while the latter shows the time that the latest
  successfully confirmed entry waited for the quorum to gather (gh-9918).
* A replica deleted from the `_cluster` space now applies its own deletion and
  does not try to rejoin (gh-10088).
* Made all of the system spaces synchronous by default when the synchronous
  queue is claimed, i.e., `box.info.synchro.queue.owner ~= 0`. Added a
  `box_consider_system_spaces_synchronous` backward compatibility option to
  control this behavior. Added a new read-only `state` subtable for the Lua
  space object returned from the `box.space` registry. This subtable has an
  `is_sync` field that reflects the effective state of replication for system
  spaces. For user spaces, this field will always mirror the `is_sync` option
  set for the space (gh-9723).

### Lua

* Added support for the `level` argument in `box.error` and `box.error.new`
  when an error is created using an error code (gh-9792).
* Added `box.error.is` that checks whether its argument is a `box.error`.
* Introduced a standard Lua way to get the length of `key_def` (gh-10111).
* Introduced Lua implementation of the protobuf encoder (gh-9844).
* Deprecated using `cdata` values with `histogram:observe` from the `metrics` module.
* Updated memtx metrics descriptions from the `metrics` module to be consistent.
* Added new metrics to the `metrics` module: `tnt_memtx_tuples_data_total`,
  `tnt_memtx_tuples_data_read_view`, `tnt_memtx_tuples_data_garbage`,
  `tnt_memtx_index_total`, `tnt_memtx_index_read_view`, `tnt_vinyl_memory_tuple`,
  `tnt_config_alerts`, `tnt_config_status`.
* Introduced the `tarantool.build.tzdata_version` option to get
  the `tzdata` version.

#### Popen

* Eliminated polling in `<popen handle>:wait()`, so now it reacts to SIGCHLD
  faster and performs less unnecessary work (gh-4915).
* Added the `timeout` parameter for `<popen handle>:wait()` (gh-4916).

### Config

* Expose configuration status from `box.info.config` (gh-10044).
* Expose the `experimental.config.utils.schema` built-in module to the public
  API (gh-10117).
* `<schema object>:get()` now supports indexing values inside the `any` type
  (gh-10205).
* `config:get()` now supports indexing values inside `app.cfg.<key>` and
  `roles_cfg.<key>` (gh-10205).
* Allow to delete a field or set it to `box.NULL` using the `<schema
  object>:set()` method (gh-10190, gh-10193, gh-10194).
* Now it is possible to grant access to a specific global Lua function
  in the declarative configuration (gh-10304).

### Connpool

* The `connpool.filter()` and `connpool.call()` functions now support
  filtering by the `sharding.roles` option (gh-10318).

### Console

* Interactive console now performs non-local assignments to a per-session
  variable scope if the `console_session_scope_vars` compat option is set to
  `new` (gh-9985).
* Interactive console now automatically adds frequently used built-in modules
  into the initial environment if the `console_session_scope_vars` compat
  option is set to `new` (gh-9986).

## Bugs fixed

### Core

* Added details for DML specific errors (gh-7223).
* Now fully-temporary spaces DDL does not abort concurrent purely remote
  (applier) transactions, and DDL in purely remote transactions does not abort
  concurrent fully-temporary spaces transactions (gh-9720).

### Replication

* Fixed a bug that allowed asynchronous transactions from a replica deleted from
  the cluster to arrive on the remaining cluster members (gh-10266).
* Significantly improved replication performance by batching rows to be sent
  before dispatch. Batch size in bytes may be controlled by a new tweak
  `xrow_stream_flush_size` (default is 16 kb) (gh-10161).

### Lua

#### Popen

* Defined the behavior of in-progress `<popen handle>:wait()` when
  `<popen handle>:close()` is called in another fiber: now it returns an error
  instead of accessing a freed memory and, likely, hanging forever (gh-7653).
